<?php

class MageOne_RateLimit_Model_Cron
{
    protected $configProvider;

    public function __construct()
    {
        $this->configProvider = new MageOne_RateLimit_Model_ConfigProvider();
    }

    public function cleanupLogEntries()
    {
        $rateLimitConfig = $this->configProvider->getRateLimitConfig();

        foreach ($rateLimitConfig as $routeKey => $routeData) {
            if (isset($routeData['route']) && isset($routeData['time_window_in_seconds'])) {
                $route = $routeData['route'];
                $timeWindow = (int) $routeData['time_window_in_seconds'];

                $olderThan = date('Y-m-d H:i:s', strtotime("-$timeWindow seconds"));

                $collection = Mage::getModel('mageone_ratelimit/log')->getCollection()
                    ->addFieldToFilter('route', ['eq' => $route])
                    ->addFieldToFilter('created_at', ['lt' => $olderThan]);

                $idsToDelete = $collection->getAllIds();

                if (!empty($idsToDelete)) {
                    $resource = Mage::getSingleton('core/resource');
                    $connection = $resource->getConnection('core_write');

                    $connection->beginTransaction();

                    try {
                        $table = $resource->getTableName('mageone_ratelimit/log');
                        $where = ['id IN (?)' => $idsToDelete];
                        $connection->delete($table, $where);

                        $connection->commit();
                        echo "Entries for route $route deleted successfully." . PHP_EOL;
                    } catch (Exception $e) {
                        $connection->rollBack();
                        Mage::logException($e);
                        echo "Failed to delete entries for route $route: " . $e->getMessage() . PHP_EOL;
                    }
                } else {
                    echo "No entries found to delete for route $route." . PHP_EOL;
                }
            }
        }
    }
}
