<?php

class MageOne_RateLimit_Model_RateLimiter
{
    protected $response = null;

    protected $configProvider;

    public function __construct()
    {
        $this->configProvider = new MageOne_RateLimit_Model_ConfigProvider();
    }

    public function validateRequest($path)
    {
        $clientIpAddress = Mage::helper('core/http')->getRemoteAddr();

        $config = $this->configProvider->getRateLimitConfig($path);

        if ($config === null || !$config['enabled']) {
            return true;
        }

        $maxQueries = $config['max_queries'];
        $timeWindow = $config['time_window'];

        $logEntry = Mage::getModel('mageone_ratelimit/log');
        $logEntry->setRoute($path);
        $logEntry->setRemoteAddr($clientIpAddress);
        $logEntry->setCreatedAt(now());
        $logEntry->save();

        if ($maxQueries == 0 || $timeWindow == 0) {
            return true;
        }

        $olderThan = date('Y-m-d H:i:s', strtotime("-$timeWindow seconds"));

        $logEntryCollection = Mage::getModel('mageone_ratelimit/log')->getCollection()
            ->addFieldToFilter('route', ['eq' => $path])
            ->addFieldToFilter('remote_addr', ['eq' => $clientIpAddress])
            ->addFieldToFilter('created_at', ['gt' => $olderThan]);

        if (count($logEntryCollection) > $maxQueries) {
            $this->response = Mage::app()->getResponse();
            $this->response->setHttpResponseCode(429);
            $this->response->setBody(json_encode([
                'error' => 'Too many requests',
                'message' => 'You have exceeded the rate limit. Please try again later.'
            ]));
            $this->response->sendResponse();
            exit;
        }

        return true;
    }
}
